@@ -32,6 +32,7 @@ from typing import (
 import numpy as np
 from qctrlcommons.exceptions import QctrlArgumentsValueError
 from qctrlcommons.graph import Graph
+from qctrlcommons.node.node_data import Pwc
 from qctrlcommons.preconditions import (
     check_argument,
     check_argument_positive_integer,
@@ -47,7 +48,7 @@ class OptimizableCoefficient(ABC):
     """
 
     @abstractmethod
-    def get_pwc(self, graph, gate_duration, name):
+    def get_pwc(self, graph: Graph, gate_duration: float, name) -> Pwc:
         """
         Return a Pwc representation of the optimizable coefficient.
         """
@@ -67,6 +68,15 @@ class RealOptimizableConstant(OptimizableCoefficient):
         The minimum value that the coefficient can take.
     max : float
         The maximum value that the coefficient can take.
+
+    See Also
+    --------
+    :class:`.superconducting.ComplexOptimizableConstant` :
+        Class describing complex optimizable constant coefficient.
+    :class:`.superconducting.ComplexOptimizableSignal` :
+        Class describing complex optimizable piecewise-constant coefficients.
+    :class:`.superconducting.RealOptimizableSignal` :
+        Class describing real optimizable piecewise-constant coefficients.
     """
 
     min: float
@@ -79,7 +89,7 @@ class RealOptimizableConstant(OptimizableCoefficient):
             {"min": self.min, "max": self.max},
         )
 
-    def get_pwc(self, graph, gate_duration, name):
+    def get_pwc(self, graph: Graph, gate_duration: float, name: str) -> Pwc:
         value = graph.optimization_variable(1, self.min, self.max)[0]
         value.name = name
         return graph.constant_pwc(constant=value, duration=gate_duration)
@@ -98,6 +108,15 @@ class ComplexOptimizableConstant(OptimizableCoefficient):
         The minimum value that the modulus of the coefficient can take.
     max_modulus : float
         The maximum value that the modulus of the coefficient can take.
+
+    See Also
+    --------
+    :class:`.superconducting.ComplexOptimizableSignal` :
+        Class describing complex optimizable piecewise-constant coefficients.
+    :class:`.superconducting.RealOptimizableConstant` :
+        Class describing real optimizable constant coefficients.
+    :class:`.superconducting.RealOptimizableSignal` :
+        Class describing real optimizable piecewise-constant coefficients.
     """
 
     min_modulus: float
@@ -110,7 +129,7 @@ class ComplexOptimizableConstant(OptimizableCoefficient):
             {"min_modulus": self.min_modulus, "max_modulus": self.max_modulus},
         )
 
-    def get_pwc(self, graph, gate_duration, name):
+    def get_pwc(self, graph: Graph, gate_duration: float, name: str) -> Pwc:
         mod = graph.optimization_variable(1, self.min_modulus, self.max_modulus)[0]
         phase = graph.optimization_variable(1, 0, 2 * np.pi, True, True)[0]
         value = graph.multiply(mod, graph.exp(1j * phase), name=name)
@@ -133,6 +152,15 @@ class RealOptimizableSignal(OptimizableCoefficient):
         The minimum value that the signal can take at each segment.
     max : float
         The maximum value that the signal can take at each segment.
+
+    See Also
+    --------
+    :class:`.superconducting.ComplexOptimizableConstant` :
+        Class describing complex optimizable constant coefficient.
+    :class:`.superconducting.ComplexOptimizableSignal` :
+        Class describing complex optimizable piecewise-constant coefficients.
+    :class:`.superconducting.RealOptimizableConstant` :
+        Class describing real optimizable constant coefficients.
     """
 
     count: int
@@ -149,7 +177,7 @@ class RealOptimizableSignal(OptimizableCoefficient):
             {"min": self.min, "max": self.max},
         )
 
-    def get_pwc(self, graph, gate_duration, name):
+    def get_pwc(self, graph: Graph, gate_duration: float, name: str) -> Pwc:
         values = graph.optimization_variable(self.count, self.min, self.max)
         return graph.pwc_signal(values=values, duration=gate_duration, name=name)
 
@@ -170,6 +198,15 @@ class ComplexOptimizableSignal(OptimizableCoefficient):
         The minimum value that the modulus of the signal can take at each segment.
     max_modulus : float
         The maximum value that the modulus of the signal can take at each segment.
+
+    See Also
+    --------
+    :class:`.superconducting.ComplexOptimizableConstant` :
+        Class describing complex optimizable constant coefficient.
+    :class:`.superconducting.RealOptimizableConstant` :
+        Class describing real optimizable constant coefficients.
+    :class:`.superconducting.RealOptimizableSignal` :
+        Class describing real optimizable piecewise-constant coefficients.
     """
 
     count: int
@@ -186,7 +223,7 @@ class ComplexOptimizableSignal(OptimizableCoefficient):
             {"min_modulus": self.min_modulus, "max_modulus": self.max_modulus},
         )
 
-    def get_pwc(self, graph, gate_duration, name):
+    def get_pwc(self, graph: Graph, gate_duration: float, name: str) -> Pwc:
         mods = graph.optimization_variable(
             self.count, self.min_modulus, self.max_modulus
         )
@@ -240,6 +277,15 @@ class Transmon:
         The identifier of the transmon that is used to link interaction terms to this transmon.
         Defaults to "transmon".
 
+    See Also
+    --------
+    :class:`.superconducting.Cavity` :
+        Class describing cavities in superconducting systems.
+    :class:`.superconducting.TransmonCavityInteraction` :
+        Class describing interactions between a transmon and a cavity.
+    :class:`.superconducting.TransmonTransmonInteraction` :
+        Class describing interactions between two transmons.
+
     Notes
     -----
     The Hamiltonian for the transmon is defined as
@@ -299,6 +345,15 @@ class Cavity:
         The identifier of the cavity that is used to link interaction terms to this cavity.
         Defaults to "cavity".
 
+    See Also
+    --------
+    :class:`.superconducting.CavityCavityInteraction` :
+        Class describing interactions between two cavities.
+    :class:`.superconducting.Transmon` :
+        Class describing transmons in superconducting systems.
+    :class:`.superconducting.TransmonCavityInteraction` :
+        Class describing interactions between a transmon and a cavity.
+
     Notes
     -----
     The Hamiltonian for the cavity is defined as
@@ -348,6 +403,11 @@ class TransmonTransmonInteraction:
         The effective coupling between the two transmons, :math:`g`.
         If not provided, it defaults to no effective coupling term.
 
+    See Also
+    --------
+    :class:`.superconducting.Transmon` :
+        Class describing transmons in superconducting systems.
+
     Notes
     -----
     The Hamiltonian for the interaction is defined as
@@ -379,12 +439,12 @@ class TransmonCavityInteraction:
     dispersive_shift : real or np.ndarray or RealOptimizableSignal or \
             RealOptimizableConstant, optional
         The dispersive shift between the transmon and the cavity, :math:`\\chi`.
-        If not provided, it defaults to no dispersive shift term.
+        You must provide either a dispersive shift or a Rabi coupling.
     rabi_coupling : real or complex or np.ndarray or RealOptimizableSignal or \
             RealOptimizableConstant or ComplexOptimizableSignal or \
             ComplexOptimizableConstant, optional
         The strength of the Rabi coupling between the transmon and the cavity, :math:`\\Omega`.
-        If not provided, it defaults to no Rabi coupling term.
+        You must provide either a dispersive shift or a Rabi coupling.
     transmon_name : str, optional
         The name identifying the transmon in the interaction.
         Defaults to "transmon".
@@ -392,27 +452,41 @@ class TransmonCavityInteraction:
         The name identifying the cavity in the interaction.
         Defaults to "cavity".
 
+    See Also
+    --------
+    :class:`.superconducting.Cavity` :
+        Class describing cavities in superconducting systems.
+    :class:`.superconducting.Transmon` :
+        Class describing transmons in superconducting systems.
+
     Notes
     -----
     The Hamiltonian for the interaction is defined as
 
-    .. math::
-        H_\\mathrm{transmon-cavity} =
-            \\chi a^\\dagger a b^\\dagger b
-            + \\frac{1}{2} \\left(\\Omega a b^\\dagger + H.c.\\right) \\;,
+    .. math:: H_\\mathrm{transmon-cavity} = \\chi a^\\dagger a b^\\dagger b \\;,
+
+    or as
+
+    .. math:: H_\\mathrm{transmon-cavity} = \\Omega a b^\\dagger + H.c. \\;,
 
     where :math:`H.c.` indicates the Hermitian conjugate.
-    All coefficients in the Hamiltonian are optional,
-    and you should only pass those relevant to your system.
     """
 
     dispersive_shift: Optional[RealCoefficient] = None
     rabi_coupling: Optional[Coefficient] = None
-    transmon_name: Optional[str] = "transmon"
-    cavity_name: Optional[str] = "cavity"
+    transmon_name: str = "transmon"
+    cavity_name: str = "cavity"
 
     def __post_init__(self):
         _check_argument_real_coefficient(self.dispersive_shift, "dispersive shift")
+        check_argument(
+            (self.dispersive_shift is None) ^ (self.rabi_coupling is None),  # xor
+            "You must provide either a dispersive shift or a Rabi coupling.",
+            {
+                "dispersive_shift": self.dispersive_shift,
+                "rabi_coupling": self.rabi_coupling,
+            },
+        )
 
 
 @expose(Namespace.SUPERCONDUCTING)
@@ -430,6 +504,11 @@ class CavityCavityInteraction:
         The cross-Kerr coefficient between the two cavities, :math:`K_{12}`.
         If not provided, it defaults to no cross-Kerr term.
 
+    See Also
+    --------
+    :class:`.superconducting.Cavity` :
+        Class describing cavities in superconducting systems.
+
     Notes
     -----
     The Hamiltonian for the interaction is defined as
@@ -472,23 +551,26 @@ _QHOOps = namedtuple("_QHOOps", ["a", "adag", "n"])
 
 def _create_qho_operators(
     graph, transmons: List[Transmon], cavities: List[Cavity]
-) -> Tuple[Dict[str, _QHOOps], Dict[str, _QHOOps]]:
+) -> Dict[str, _QHOOps]:
     """
-    Creates the creation, annihilation, and number operators for transmons and cavities.
+    Create the creation, annihilation, and number operators for transmons and cavities.
 
-    Returns a dictionary whose keys are the subsystem names and whose values are _QHOOps
-    namedtuples (with field names a, adag, and n) with the operators of each subsystem.
+    This function returns a dictionary whose keys are the subsystem names and whose values are
+    _QHOOps namedtuples (with field names a, adag, and n) with the operators of each subsystem.
     """
 
     # Dimension of the identity in the Kronecker product before and after the operator.
     pre_dim = 1
-    post_dim = np.prod([system.dimension for system in transmons + cavities])
+    # Joining two list here with + mutates the list type, which mypy doesn't like,
+    # see https://github.com/python/mypy/issues/3933
+    post_dim = np.prod([system.dimension for system in transmons + cavities])  # type: ignore
 
-    tri_kron = lambda op1, op2, op3: graph.kron(graph.kron(op1, op2), op3)
+    def tri_kron(op1, op2, op3):
+        return graph.kron(graph.kron(op1, op2), op3)
 
     # Define operators for each subsystem.
     operators = {}
-    for system in transmons + cavities:
+    for system in transmons + cavities:  # type: ignore
         dim = system.dimension
         post_dim //= dim
         pre_eye = graph.tensor(np.eye(pre_dim))
@@ -513,17 +595,17 @@ def _validate_physical_system_inputs(
     List[CavityCavityInteraction],
 ]:
     """
-    Validates the type of the subsystems and interactions.
-    Validates the subsystem names in the interactions.
+    Validate the type of the subsystems and interactions,
+    and the subsystem names in the interactions.
 
-    Returns three lists containing
+    This function returns three lists containing
     (1) all the transmon-transmon interactions,
     (2) all the transmon-cavity interactions, and
     (3) all the cavity-cavity interactions.
     """
 
     check_argument(
-        len(transmons + cavities) > 0,
+        len(transmons) + len(cavities) > 0,
         "At least one transmon or cavity must be provided.",
         {"transmons": transmons, "cavities": cavities},
     )
@@ -574,11 +656,9 @@ def _validate_physical_system_inputs(
         intx for intx in interactions if isinstance(intx, CavityCavityInteraction)
     ]
     check_argument(
-        len(
-            transmon_transmon_interactions
-            + transmon_cavity_interactions
-            + cavity_cavity_interactions
-        )
+        len(transmon_transmon_interactions)
+        + len(transmon_cavity_interactions)
+        + len(cavity_cavity_interactions)
         == len(interactions),
         "Each element in interactions must be a TransmonTransmonInteraction, "
         "a TransmonCavityInteraction, or a CavityCavityInteraction object.",
@@ -662,13 +742,14 @@ def _create_superconducting_hamiltonian(
     cavities: List[Cavity],
     interactions: List[Interaction],
     gate_duration: float,
-    cutoff_frequency: float,
+    cutoff_frequency: Optional[float],
     sample_count: int,
 ):
     r"""
-    Creates the Hamiltonian of a system composed of a single transmon and one or more cavities.
-    Returns the Hamiltonian as a Pwc node and a list with the names of the optimizable nodes
-    that have been added to the graph.
+    Create the Hamiltonian of a system composed of transmons and cavities.
+
+    This function returns the Hamiltonian as a Pwc node and a list with the names of
+    the optimizable nodes that have been added to the graph.
 
     Parameters
     ----------
@@ -682,7 +763,7 @@ def _create_superconducting_hamiltonian(
         List of objects containing the physical information about the interactions in the system.
     gate_duration : float
         The duration of the gate.
-    cutoff_frequency : float
+    cutoff_frequency : float or None
         The cutoff frequency of a linear sinc filter to be applied to the piecewise-constant
         signals you provide for the coefficients. If None, the signals are not filtered.
     sample_count : int
@@ -698,9 +779,9 @@ def _create_superconducting_hamiltonian(
         then the names of the filtered PWC nodes are also included.
     """
 
-    def convert_to_pwc(coefficient: Coefficient, real_valued: bool, name: str):
+    def convert_to_pwc(coefficient: Coefficient, real_valued: bool, name: str) -> Pwc:
         """
-        Returns the Pwc representation of a coefficient.
+        Return the Pwc representation of a coefficient.
         """
 
         def filter_signal(signal):
@@ -764,7 +845,7 @@ def _create_superconducting_hamiltonian(
     operators = _create_qho_operators(graph, transmons, cavities)
 
     # Create nested dictionary structure containing information for the different Hamiltonian terms.
-    hamiltonian_info = {}
+    hamiltonian_info: Dict[str, Any] = {}
 
     # Add transmon terms.
     for transmon in transmons:
@@ -810,13 +891,9 @@ def _create_superconducting_hamiltonian(
         transmon_1_ops = operators[name_1]
         transmon_2_ops = operators[name_2]
         key = f"{name_1}_{name_2}_interaction"
-        if tt_interaction.effective_coupling is not None:
-            coefficient = 2.0 * tt_interaction.effective_coupling
-        else:
-            coefficient = None
         hamiltonian_info[f"{key}.effective_coupling"] = {
-            "coefficient": coefficient,
-            "operator": transmon_1_ops.a @ transmon_2_ops.adag,
+            "coefficient": tt_interaction.effective_coupling,
+            "operator": 2 * transmon_1_ops.a @ transmon_2_ops.adag,
             "is_hermitian": False,
         }
 
@@ -832,7 +909,7 @@ def _create_superconducting_hamiltonian(
         }
         hamiltonian_info[f"{key}.rabi_coupling"] = {
             "coefficient": tc_interaction.rabi_coupling,
-            "operator": transmon_ops.adag @ cavity_ops.a,
+            "operator": 2 * transmon_ops.adag @ cavity_ops.a,
             "is_hermitian": False,
         }
 
@@ -862,7 +939,7 @@ def _create_superconducting_hamiltonian(
 
     # Build the Hamiltonian from the different terms.
     hamiltonian_terms = []
-    optimizable_node_names = []  # filled up by convert_to_pwc
+    optimizable_node_names: List[str] = []  # filled up by convert_to_pwc
 
     for name, info in hamiltonian_info.items():
         if info["coefficient"] is not None:
@@ -884,7 +961,7 @@ def _create_superconducting_hamiltonian(
 
 def _extract_output(item):
     """
-    Converts a non-batched item from a qctrl.types.graph/optimization.Result.output
+    Convert a non-batched item from a qctrl.types.graph/optimization.Result.output
     dictionary into a NumPy array.
 
     If the item is a dictionary (corresponds to a tensor), then its "value" is returned.
@@ -914,7 +991,7 @@ def simulate(
     initial_state: Optional[np.ndarray] = None,
 ):
     """
-    Simulates a system composed of transmons and cavities.
+    Simulate a system composed of transmons and cavities.
 
     Parameters
     ----------
@@ -967,6 +1044,11 @@ def simulate(
                 as an array of shape ``(T, D)``.
                 This is only returned if you provide an initial state.
 
+    See Also
+    --------
+    :func:`.superconducting.optimize` :
+        Find optimal pulses or parameters for a system composed of transmons and cavities.
+
     Notes
     -----
     The Hamiltonian of the system is of the form
@@ -993,9 +1075,13 @@ def simulate(
             \\otimes \\ldots
 
     The system dimension `D` is then the product of all transmon and cavity dimensions.
+
+    See the `superconducting systems namespace classes
+    <https://docs.q-ctrl.com/boulder-opal/references/qctrl/Toolkits/superconducting.html#classes>`_
+    for a list of the relevant objects to describe transmons and cavities.
     """
     system_dimension = np.prod(
-        [system.dimension for system in transmons + cavities], dtype=int
+        [system.dimension for system in transmons + cavities], dtype=int  # type: ignore
     )
 
     if initial_state is not None:
@@ -1084,7 +1170,7 @@ def optimize(
     optimization_count: int = 5,
 ) -> Dict[str, Any]:
     """
-    Finds optimal pulses or parameters for a system composed of transmons and cavities,
+    Find optimal pulses or parameters for a system composed of transmons and cavities,
     in order to achieve a target state or implement a target operation.
 
     At least one of the terms in the transmon, cavities, or interaction Hamiltonians
@@ -1171,6 +1257,10 @@ def optimize(
                 as an array of shape ``(T, D)``.
                 This is only returned if you provide an initial state.
 
+    See Also
+    --------
+    :func:`.superconducting.simulate` : Simulate a system composed of transmons and cavities.
+
     Notes
     -----
     The Hamiltonian of the system is of the form
@@ -1220,6 +1310,10 @@ def optimize(
                 {\\mathrm{Tr} (U_\\mathrm{target}^\\dagger U(t_\\mathrm{gate}))}
                 {\\mathrm{Tr} (U_\\mathrm{target}^\\dagger U_\\mathrm{target})}
             \\right|^2 .
+
+    See the `superconducting systems namespace classes
+    <https://docs.q-ctrl.com/boulder-opal/references/qctrl/Toolkits/superconducting.html#classes>`_
+    for a list of the relevant objects to describe subsystems and optimizable coefficients.
     """
 
     check_argument(
@@ -1229,7 +1323,7 @@ def optimize(
     )
 
     system_dimension = np.prod(
-        [system.dimension for system in transmons + cavities], dtype=int
+        [system.dimension for system in transmons + cavities], dtype=int  # type: ignore
     )
 
     if initial_state is not None:
@@ -1341,7 +1435,7 @@ def optimize(
     )
 
     # Retrieve results and build output dictionary.
-    result_dict = {"optimized_values": {}, "sample_times": sample_times}
+    result_dict: Dict[str, Any] = {"optimized_values": {}, "sample_times": sample_times}
 
     for key in optimizable_node_names:
         result_dict["optimized_values"][key] = _extract_output(
